=head1 NAME

Bio::Polloc - Perl library for Polymorphic Locy analyses

=head1 AUTHOR - Luis M. Rodriguez-R

Email lrr at cpan dot org

=head1 LICENSE

This package is licensed under the Artistic License - see LICENSE.txt

=cut

package Bio::Polloc;

use strict;
use base qw(Bio::Polloc::Polloc::Root);

our $VERSION = 1.0503; # [a-version] from Bio::Polloc::Polloc::Version


1;

